#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2004-2005 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: pylucene_impl.py 381 2005-04-28 17:09:04Z vslavik $
#
#  Fulltext indexer using PyLucene -- implementation

import sys
import PyLucene

from __init__ import FulltextIndexer, QueryParsingError # ugly, I admit

try:
    ParsingException = PyLucene.JavaError # PyLucene >= 0.9.8
except AttributeError:
    ParsingException = ValueError # PyLucene < 0.9.8

try:
    PyLuceneThreadClass = PyLucene.PythonThread
except AttributeError:
    # old PyLucene, in this case we're running out-of-process and thread
    # class is not used for anything
    PyLuceneThreadClass = None

def getPyLuceneVersion():
    verstr = PyLucene.VERSION.split('.')
    return (int(verstr[0]), int(verstr[1]), int(verstr[2]))

class PyLuceneIndexer(FulltextIndexer):
    """PyLucene-based fulltext indexer - the meal."""

    # PyLucene requires that threads that run PyLucene/GCJ code derive
    # from PythonThread:
    ThreadClass = PyLuceneThreadClass

    def __init__(self):
        self.writer = None
    
    def getNameAndVersion(self):
        return 'PyLucene %s using Lucene %s' % (
                    PyLucene.VERSION, PyLucene.LUCENE_VERSION)
    
    def search(self, directory, query):
        dir = PyLucene.FSDirectory.getDirectory(directory, False)
        searcher = PyLucene.IndexSearcher(dir)
        analyzer = PyLucene.StandardAnalyzer()
        
        try:
            # this is unfortunate, but earlier PyLucene versions didn't
            # have a way to set the default operator, so we have to interpret
            # the query differently under new and old versions:
            if getPyLuceneVersion() >= (0,9,5):
                qp = PyLucene.QueryParser('contents', analyzer)
                qp.setOperator(PyLucene.QueryParser.DEFAULT_OPERATOR_AND)
                queryobj = qp.parseQuery(query)
            else:
                queryobj = PyLucene.QueryParser.parse(query,
                                                      'contents', analyzer)
                
        except ParsingException, e:
            if ParsingException == ValueError:
                # we need to transform the ugly string that contains Java
                # exception name and backtrace into something human-friendly:
                msg = str(e)
                prefix = 'org.apache.lucene.queryParser.ParseException: '
                if msg.startswith(prefix):
                    msg = msg[len(prefix):]   # removes exception name
                    msg = msg.split('\n')[0]  # removes backtrace
                raise QueryParsingError(query, msg)
            else:
                raise QueryParsingError(query,
                                        e.getJavaException().getMessage())
        
        hits = searcher.search(queryobj)
        
        results = []
        for hit in xrange(hits.length()):
            doc = hits.doc(hit)
            results.append(FulltextIndexer.Result(
                            doc.get('title'), doc.get('url'), hits.score(hit)))
        return results
 
    def startIndexing(self, directory):
        store = PyLucene.FSDirectory.getDirectory(directory, True)
        self.writer = PyLucene.IndexWriter(store,
                                           PyLucene.StandardAnalyzer(),
                                           True)

    def indexDocument(self, url, data):
        doc = PyLucene.Document()
        doc.add(PyLucene.Field.UnIndexed('url', url)) # just store
        for field in data:
            if field == 'title':
                # analyze, index, store (we need to retrieve the title)
                doc.add(PyLucene.Field.Text(field, data[field]))
            else:
                # analyze, index, don't store
                doc.add(PyLucene.Field.UnStored(field, data[field]))
        self.writer.addDocument(doc)

    def stopIndexing(self):
        self.writer.optimize()
        self.writer.close()
        self.writer = None
