#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2005 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: provider.py 348 2005-02-22 23:05:36Z vslavik $
#
#  DevHelp documentation provider
#

import os.path

import cache, book
import providers
from providers.html.provider import HtmlProviderBase


# ----------------------------------------------------------------------------
# Content
# ----------------------------------------------------------------------------

class Content:
    """Parsed content of .devhelp file."""
    def __init__(self, filename, baseurl):
        self.baseurl = baseurl
        self._parse(filename)
    
    def _parse(self, filename):
        import xml.dom.minidom
        doc = xml.dom.minidom.parse(filename).documentElement
        
        self.title = doc.getAttribute('title')
        self.homepage = self.baseurl + doc.getAttribute('link')

        # read index entries:
        indexNode = doc.getElementsByTagName('functions')
        if len(indexNode) > 0:
            indexNode = indexNode[0]
            self.index = []
            for f in [x for x in indexNode.childNodes if x.nodeName == 'function']:
                self.index.append((f.getAttribute('name'),
                               self.baseurl + f.getAttribute('link')))

        # read content tree:
        contentNode = doc.getElementsByTagName('chapters')[0]
        self.contents = book.ContentsNode(None, self.title, self.homepage)
        
        def parseChildren(parent, node):
            parent.children = []
            for sub in [x for x in node.childNodes if x.nodeName == 'sub']:
                c = book.ContentsNode(parent,
                                      sub.getAttribute('name'),
                                      self.baseurl + sub.getAttribute('link'))
                parent.children.append(c)
                parseChildren(c, sub)
        
        parseChildren(self.contents, contentNode)
        

class CachedContent(cache.DataCacheObject):
    """Cache object for content data."""

    ID = 'content'
    
    def __init__(self, book, devhelpFile, baseurl):
        self.devhelpFile = devhelpFile
        self.baseurl = baseurl
        dep = cache.FilesListDependency([devhelpFile])
        cache.DataCacheObject.__init__(self, book, CachedContent.ID, [dep])

    def doUpdate(self, filename, ctrl):
        try:
            self.setObject(Content(self.devhelpFile, self.baseurl),
                           filename=filename)
        except:
            # FIXME: report the error
            return False
        return True


# ----------------------------------------------------------------------------
# DevHelpProvider
# ----------------------------------------------------------------------------

class DevHelpProvider(HtmlProviderBase):
    def __init__(self):
        HtmlProviderBase.__init__(self)
    
    def getFileDescription(self):
        return _('DevHelp books')

    def getFileMasks(self):
        return ['*.devhelp']

    def getID(self):
        return 'devhelp'
    
    def getName(self):
        return _('DevHelp')

    def getDescription(self):
        return _('Documentation in <a href="http://www.devhelp.net">DevHelp</a> book.')

    def initForBook(self, book):
        import indexer
        baseurl = self._filename2url(os.path.dirname(book.getAttr('filename')),
                                     book)
        baseurl += '/'
        content = CachedContent(book, book.getAttr('filename'), baseurl)
        c = cache.get(book)
        c.addObject(content)
        c.addObject(indexer.FulltextCacheObject(book, deps=[content]))

    def getHomeURL(self, book):
        c = cache.get(book).get(CachedContent.ID, forceUpdate=True)
        return c.getObject().homepage

    def getContents(self, book):
        c = cache.get(book).get(CachedContent.ID, forceUpdate=True)
        return c.getObject().contents


providers.register(DevHelpProvider())
