"""Yahoo Search Web Services

This module implements a set of classes and functions to work with the
Yahoo Search Web Services. All results from these services are properly
formatted XML, and this package facilitates for proper parsing of these
result sets. Some of the features include:

    * Extendandable API, with replaceable backend XML parsers, and
      I/O interface.
    * Type and value checking on search parameters, including
      automatic type conversion (when appropriate and possible)
    * Flexible return format, including DOM objects, or fully
      parsed result objects


You can either instantiate a search object directly, or use the factory
function create_search() in this module (see below). The supported classes
of searches are:
    
    VideoSearch	- Video Search
    ImageSearch	- Image Search
    WebSearch	- Web Search
    NewsSearch	- News Search
    LocalSearch	- Local Search

    RelatedSuggestion	- Web Search Related Suggestion
    SpellingSuggestion	- Web Search Spelling Suggestion


The different sub-classes of Search supports different sets of query
parameters. This matrix gives a good idea of what each search class accepts:

                Web   Image  Video  News   Local  Related  Spelling
               -----  -----  -----  -----  -----  -------  --------
    query       [X]    [X]    [X]    [X]    [X]     [X]       [X]
    type        [X]    [X]    [X]    [X]     .       .         .
    results     [X]    [X]    [X]    [X]    [X]     [X]        .
    start       [X]    [X]    [X]    [X]    [X]      .         .

    format      [X]    [X]    [X]     .      .       .         .
    adult_ok    [X]    [X]    [X]     .      .       .         .
    similar_ok  [X]     .      .      .      .       .         .
    language    [X]     .      .      .     [X]      .         .
    sort         .      .      .     [X]     .       .         .

    radius       .      .      .      .     [X]      .         .
    street       .      .      .      .     [X]      .         .
    city         .      .      .      .     [X]      .         .
    state        .      .      .      .     [X]      .         .
    zip          .      .      .      .     [X]      .         .
    location     .      .      .      .     [X]      .         .


Each of these parameter is implemented as an attribute for the supported
classes. For example, you can set a parameter with:

    from yahoo.search.webservices import WebSearch

    app_id = "something"
    srch = WebSearch(app_id)
    srch.query = "Leif Hedstrom"
    srch.results = 40

or, if you are using the factory function:
    
    from yahoo.search.webservices import create_search

    app_id = "something"
    srch = create_search("Web", app_id, query="Leif Hedstrom", results=40)

or, the last alternative, a combination of the previous two:

    from yahoo.search.webservices import WebSearch

    app_id = "something"
    srch = WebSearch(app_id, query="Leif Hedstrom", results=40)

To retrieve a certain parameter value, simply access it as any normal
attribute:

    print "Searched for ", srch.query


For more information on these parameters, and their allowed values, please
see the official Yahoo Search Services documentation (XXX missing URL?)

Once the webservice object has been created, you can retrieve a parsed
object (typically a DOM object) using the get_results() method:

    dom = srch.get_results()

This DOM object contains all results, and can be used as is. For easier
use of the results, you can use the built-in results factory, which will
traverse the entire DOM object, and create a list of results objects.

    results = srch.parse_results(dom)

or, by using the implicit call to get_results():

    results = srch.parse_results()
    
The default XML parser and results factories should be adequate for most
users, so use the parse_results() when possible. However, both the XML
parser and the results parser can easily be overriden.


EXAMPLE:
    #!/usr/bin/python

    import sys
    from yahoo.search.webservices import create_search

    service = argv[1]
    query = " ".join(sys.argv[2:])
    app_id = "something"
    x = create_search(service, app_id, query=query, results=5)
    if x is None:
        x = create_search("Web", app_id, query=query, results=5)

    dom = srch.get_results()
    results = srch.parse_results(dom)

    for res in results:
        url = res.Url
        summary = res["Summary"]
        print "%s -> %s" (summary, url)
"""


import domparsers
import debug

import urllib
import urllib2
import types
import string
import re

__version__ = "$Revision: 1.5 $"
__author__ = "Leif Hedstrom <leif@ogre.com>"
__date__ = "Mon Feb 28 09:55:16 PST 2005"


#
# List of all supported languages
#
LANGUAGES = ('default', 'ar', 'bg', 'ca', 'szh', 'tzh', 'hr', 'cs', 'da', 'nl',
             'en', 'et', 'fi', 'fr', 'de', 'el', 'he', 'hu', 'is', 'id', 'it',
             'ja', 'ko', 'lv', 'lt', 'no', 'fa', 'pl', 'pt', 'ro', 'ru', 'sk',
             'sr', 'sl', 'es', 'sv', 'th', 'tr')

APPID_REGEX = "^[a-zA-Z0-9 _()\[\]*+\-=,.:\\\@]{8,40}$"

#
# Exceptions and error handling
#
class Error(Exception):
    """Base class for all Yahoo Web Services exceptions."""

class ParameterError(Error):
    """A parameter is missing, or has bad value"""
    pass

class ServerError(Error):
    """The Yahoo server is unavailable."""
    pass

class ClassError(Error):
    """This can only occur if the APIs aren't installed or configured
    properly. If it happens, please contact the author."""

class SearchError(Error):
    """An exception/error occured in the search."""
    def __init__(self, err):
        # ToDo XXX: Do we need to call Error.__init() here ?
        self.msg = "unknown error"
        for line in err.readlines():
            start = line.find("<Message>")
            if start > -1:
                stop = line.find("</Message>")
                if stop > -1:
                    self.msg = line[start+9:stop]

    def __str__(self):
        return self.msg


#
# First a couple of base classes for the Search services. Most of them
# are almost identical, so good candidates to sub-class one of these.
#
class _Search(debug.Debuggable):
    """Yahoo Search WebService - base class

    This class implements the core functionality of all Yahoo Search
    Services.
    """
    NAME = "Search"
    SERVICE = "Search"
    PROTOCOL = "http"
    SERVER = "api.search.yahoo.com"
    VERSION = "V1"
    _NEXT_QID = 1
    _RESULT_FACTORY = None

    def __init__(self, app_id, opener=None, xml_parser=None,
                 result_factory=None, debug=0, **args):
        """The app_id is a required argument, the Yahoo search services
        will not accept requests without a proper app_id. A valid app_id
        is a combination of 8 - 40 characters, matching the regexp
        "^[a-zA-Z0-9 _()\[\]*+\-=,.:\\\@]{8,40}$"). Please visit
        http://developer.yahoo.net/ to request an App ID for your own
        software or application.
            
        Four optional arguments can also be passed to the constructor:
        
            opener         - Opener for urllib2
            xml_parser     - Function to parse XML (default: minidom)
            result_factory - Result factory class (default: none)
            debug          - Debug level (if any)

        All other "named" arguments are passed into as a dictionary to the
        set_params() method.

        The result factory is specific to the particular web service used,
        e.g. the different Yahoo Search services will each implement their
        own factory class.

        Both of these settings can be controlled via their respective
        install method (see below).
        """
        super(_Search, self).__init__(debug)
        self._service = {"name" : self.NAME,
                         "protocol" :  self.PROTOCOL,
                         "server" : self.SERVER,
                         "version" : self.VERSION,
                         "service" : self.SERVICE}

        self.app_id = app_id
        self._valid_params = {}
        self._urllib_opener = opener
        self._xml_parser = xml_parser
        if result_factory:
            self._result_factory = result_factory
        else: 
            self._result_factory = self._RESULT_FACTORY

        if self._xml_parser is None:
            import xml.dom.minidom
            self._xml_parser = xml.dom.minidom.parse

        self._qid = self._NEXT_QID
        self._NEXT_QID += 1

        self._init_valid_params()
        self.reset()
        if args:
            self.set_params(args)

    def _init_valid_params(self):
        err = "Yahoo Search Service class %s must implement a _init_valid_params()" % (
            self.svc_name)
        raise ClassError, err

    def reset(self):
        self._params = {}
        self._url = None

    def _get_svc_name(self):
        return self._service["name"]
    def _set_svc_name(self, value):
        self._service["name"] = value
    svc_name = property(_get_svc_name, _set_svc_name, None,
                        "Descriptive name of the service")

    def _get_svc_protocol(self):
        return self._service["protocol"]
    def _set_svc_protocol(self, value):
        self._service["protocol"] = value
    svc_protocol = property(_get_svc_protocol, _set_svc_protocol, None,
                            "Service protocol (e.g. HTTP)")

    def _get_svc_service(self):
        return self._service["service"]
    def _set_svc_service(self, value):
        self._service["service"] = value
    svc_service = property(_get_svc_service, _set_svc_service, None, "Service path")

    def _get_svc_server(self):
        return self._service["server"]
    def _set_svc_server(self, value):
        self._service["server"] = value
    svc_server = property(_get_svc_server, _set_svc_server, None,
                          "Service server name or IP")

    def _get_svc_version(self):
        return self._service["version"]
    def _set_svc_version(self, value):
        self._service["version"] = value
    svc_version = property(_get_svc_version, _set_svc_version, None,
                           "Service version string")

    def _get_app_id(self):
        return self._app_id
    def _set_app_id(self, app_id):
        if (isinstance(app_id, types.StringTypes) and
            (re.compile(APPID_REGEX).match(app_id) is not None)):
            self._app_id = app_id
        else:
            raise ValueError, "`app_id' can only contain a-zA-Z0-9 _()\[\]*+\-=,.:\\\@"
    app_id = property(_get_app_id, _set_app_id, None,
                           "Application ID (issued by Yahoo)")

    # Manage service parameters
    def set_params(self, args):
        """Set one or several query parameters from a dictionary"""
        for (param, value) in args.items():
            self.set_param(param, value)

    def get_param(self, param):
        """Get the value of a query parameter, or the default value if unset"""
        if not self._valid_params.has_key(param):
            err = "`%s' is not a valid parameter for `%s'" % (param,
                                                              self._service["name"])
            raise ParameterError, err
        if self._params.has_key(param):
            return self._params[param]
        else:
            return self._valid_params[param][1]

    def set_param(self, param, value):
        """Set the value of a query parameter"""
        if not self._valid_params.has_key(param):
            err = "`%s' is not a valid parameter for `%s'" % (param,
                                                              self._service["name"])
            raise ParameterError, err
        pinfo = self._valid_params[param]
        if value is None:
            err = "`%s' can not have an undefined value" % (param)
            raise ValueError, err
        if pinfo[2] is not None:
            try:
                value = pinfo[2](value)
            except:
                pass
        if not isinstance(value, pinfo[0]):
            err = "`%s' needs a value of type %s" % (param, str(pinfo[0]))
            raise TypeError, err
        if pinfo[3] is not None:
            if not value in pinfo[3]:
                err = "`%s' can only be one of %s" % (param, str(pinfo[3]))
                raise ValueError, err
        if value != pinfo[1]:
            self._params[param] = value
        elif self._params.has_key(param):
            self._params.pop(param)

    def get_valid_params(self):
        """Return a list of all valid parameters for this search"""
        return self._valid_params.keys()

    # Manage (install) the Opener, XML parser and result factory (parser)
    def install_opener(self, opener):
        """Install a URL opener (for use with urllib2), overriding the
        default opener. This is rarely required.
        """
        self._urllib_opener = opener
        
    def install_xml_parser(self, xml_parser):
        """Install an XML parser that will be used for all results for this
        object. The parser is expected to "read" the data from the supplied
        stream argument. To uninstall the parser (e.g. to make sure we
        return raw XML data) simply call this method with an argument of
        None.
        """
        self._xml_parser = xml_parser

    def install_result_factory(self, result_factory):
        """Install a python class (not an instance!) that should be used as a
        factory for creating result(s) objects.
        """
        self._result_factory = result_factory

    # Methods working on connection handling etc.
    def get_url(self):
        """Return the URL for this request object"""
        
        params = self._params.copy()
        params.update({"appid" : self._app_id})
        self._url = "%s://%s/%s/%s/%s?%s" % (self._service["protocol"],
                                             self._service["server"],
                                             self._service["service"],
                                             self._service["version"],
                                             self._service["name"],
                                             urllib.urlencode(params))
        return self._url

    def open(self, opener=None):
        """Open a connection to the webservice server, and request the URL.
        The return value is a "stream", which can be read calling the
        read(), readline() or readlines() methods.
        """
        if opener is not None:
            urllib2.install_opener(opener)
        elif self._urllib_opener is not None:
            urllib2.install_opener(opener)

        url = self.get_url()
        self._debug_msg("Opening URL = %s", debug.DEBUG_LEVELS["HTTP"], url)

        # XXX ToDo: This needs to be properly implemented?
        try:
            resp = urllib2.urlopen(url)
        except urllib2.HTTPError, err:
            if err.code == 503:
                raise ServerError("Internal WebService error")
            else:
                raise SearchError(err)
            raise ServerError("WebService server unavailable")
        return resp

    def get_results(self, stream=None, xml_parser=None, close=True):
        """Read the stream (if provided) and either return the raw XML, or
        send the data to the provided XML parser for further processing.
        If no stream is provided, it will call the open() method using the
        default opener. The stream will be closed upon return from this
        method, unless the close=False is passed as an argument.
        """
        self._debug_msg("VALID_PARAMS = %s", debug.DEBUG_LEVELS["PARAMS"],
                        self._valid_params.keys())
        if stream is None:
            stream = self.open()
        if xml_parser is None:
            xml_parser = self._xml_parser
        res = xml_parser(stream)
        if close:
            stream.close()
        return res

    def parse_results(self, xml=None):
        """Get the result from the request, and instantiate the appropriate
        result class. This class will be populated with all the data from
        the XML object.
        """
        if self._result_factory is None:
            return None

        if xml is None:
            xml = self.get_results()
        res = self._result_factory(service=self)
        res.parse_results(xml)
        return res


    def _get_debug_level(self):
        return self._debug_level
    def _set_debug_level(self, level):
        self._debug_level = level
    debug_level = property(_get_debug_level, _set_debug_level, None,
                           "Set and modify the debug level")


#
# Basic parameters, supported by all search classes
#
class _BasicParams(object):
    """Yahoo Search WebService - common basic params

    Setup the basic CGI parameters that all (normal) search services
    supports. This is used by most services (classes) to provision for
    the basic parameters they all use.
    """
    def _init_valid_params(self):
        self._valid_params.update({
            "query" : (types.StringTypes, None, None, None),
            "results" : (types.IntType, 10, int, range(1,51)),
            "start" : (types.IntType, 1, int, None),
            })

    def _get_query(self):
        return self.get_param("query")
    def _set_query(self, value):
        self.set_param("query",value)
    query = property(_get_query, _set_query, None,
                     "The query to search for")

    def _get_results(self):
        return self.get_param("results")
    def _set_results(self, value):
        self.set_param("results", value)
    results = property(_get_results, _set_results, None,
                       "The number of results to return")

    def _get_start(self):
        return self.get_param("start")
    def _set_start(self, value):
        self.set_param("start", value)
    start = property(_get_start, _set_start, None,
"""The starting result position. This number should increment by the
value of the results argument each time the user chooses the next
page of results.""")



#
# Common search parameters, shared by several classes, but not all.
#
class _CommonParams(_BasicParams):
    """Several search services share a few non-basic parameters, so
    this sub-class of _BasicParams saves some typing.
    """
    def _init_valid_params(self):
        super(_CommonParams, self)._init_valid_params()
        self._valid_params.update({
            "type" : (types.StringTypes, "all", string.lower,
                      ("all", "any", "phrase")),
            "format" : (types.StringTypes, "all", string.lower, ("all",)),
            "adult_ok" : (types.IntType, None, int, (1,)),
            })


    def _get_type(self):
        return self.get_param("type")
    def _set_type(self, value):
        self.set_param("type", value)
    type = property(_get_type, _set_type, None,
"""The kind of search to submit:

    * all returns results with all query terms
    * any returns results with one or more of the query terms
    * phrase returns results containing the query terms as a phrase""")

    def _get_format(self):
        return self.get_param("format")
    def _set_format(self, value):
        self.set_param("format", value)
    format = property(_get_format, _set_format, None,
                      "Specifies the type of content to search for")

    def _get_adult_ok(self):
        return self.get_param("adult_ok")
    def _set_adult_ok(self, value):
        self.set_param("adult_ok", value)
    adult_ok = property(_get_adult_ok, _set_adult_ok, None,
"""The service filters out adult content by default. Enter a 1 to
allow adult content""")


#
# These are the interesting classes, one for each of the major
# search services
#
class VideoSearch(_CommonParams, _Search):
    """VideoSearch - perform a Yahoo Video Search

    This class implements the Video Search web service APIs. Allowed
    parameters are:
    
        query        - The query to search for
        type         - The kind of search to submit (all, any or phrase)
        results      - The number of results to return
        start        - The starting result position. This number should
                       increment by the value of the results argument
                       each time the user chooses the next page of results
        format       - Specifies the kind of video file to search for
        adult_ok     - The service filters out adult content by default.
                       Enter a 1 to allow adult content

    Full documentation for this service is available at:

        http://developer.yahoo.net/video/V1/videoSearch.html
    """
    NAME = "videoSearch"
    SERVICE = "VideoSearchService"
    _RESULT_FACTORY = domparsers.VideoSearch

    def _init_valid_params(self):
        super(VideoSearch, self)._init_valid_params()
        self._valid_params.update({
            "format" : (types.StringTypes, "all", string.lower,
                        ("all", "avi", "flash", "mpeg", "msmedia", "quicktime",
                         "realmedia")),
            })


class ImageSearch(_CommonParams, _Search):
    """ImageSearch - perform a Yahoo Image Search

    This class implements the Image Search web service APIs. Allowed
    parameters are:
    
        query        - The query to search for
        type         - The kind of search to submit (all, any or phrase)
        results      - The number of results to return
        start        - The starting result position. This number should
                       increment by the value of the results argument
                       each time the user chooses the next page of results
        format       - Specifies the kind of image file to search for
        adult_ok     - The service filters out adult content by default.
                       Enter a 1 to allow adult content

    Full documentation for this service is available at:

        http://developer.yahoo.net/image/V1/imageSearch.html
    """
    NAME = "imageSearch"
    SERVICE = "ImageSearchService"
    _RESULT_FACTORY = domparsers.ImageSearch

    def _init_valid_params(self):
        super(ImageSearch, self)._init_valid_params()
        self._valid_params.update({
            "format" : (types.StringTypes, "all", string.lower,
                        ("all", "bmp", "gif", "jpeg", "png")),
            })


class WebSearch(_CommonParams, _Search):
    """WebSearch - perform a Yahoo Web Search

    This class implements the Web Search web service APIs. Allowed
    parameters are:
    
        query        - The query to search for
        type         - The kind of search to submit (all, any or phrase)
        results      - The number of results to return
        start        - The starting result position. This number should
                       increment by the value of the results argument
                       each time the user chooses the next page of results
        format       - Specifies the kind of web file to search for
        adult_ok     - The service filters out adult content by default.
                       Enter a 1 to allow adult content
        similar_ok   - Specifies whether to allow multiple results with
                       similar content. Enter a 1 to allow similar content
        language     - The language the results are written in

    Full documentation for this service is available at:

        http://developer.yahoo.net/web/V1/webSearch.html
    """
    NAME = "webSearch"
    SERVICE = "WebSearchService"
    _RESULT_FACTORY = domparsers.WebSearch

    def _init_valid_params(self):
        super(WebSearch, self)._init_valid_params()
        self._valid_params.update({
            "similar_ok" : (types.IntType, None, int, (1,)),
            "format" : (types.StringTypes, "all", string.lower,
                        ("all", "html", "pdf", "ppt", "msword", "rss", "txt")),
            "language" : (types.StringTypes, "en", string.lower, LANGUAGES),
            })


    def _get_similar_ok(self):
        return self.get_param("similar_ok")
    def _set_similar_ok(self, value):
        self.set_param("similar_ok", value)
    similar_ok = property(_get_similar_ok, _set_similar_ok, None,
"""Specifies whether to allow multiple results with similar content.
Enter a 1 to allow similar content""")

    def _get_language(self):
        return self.get_param("language")
    def _set_language(self, value):
        self.set_param("language", value)
    language = property(_get_language, _set_language, None,
                        "The language the results are written in")


class RelatedSuggestion(_Search):
    """RelatedSuggestion - perform a Yahoo Web Related Suggestions search

    This class implements the Web Search Related Suggestion web service
    APIs. The only allowed parameters are:

        query        - The query to get related searches from
        results      - The number of results to return

    Full documentation for this service is available at:

        http://developer.yahoo.net/web/V1/relatedSuggestion.html
    """
    NAME = "relatedSuggestion"
    SERVICE = "WebSearchService"
    _RESULT_FACTORY = domparsers.RelatedSuggestion

    def _init_valid_params(self):
        self._valid_params.update({
            "query" : (types.StringTypes, None, None,None),
            "results" : (types.IntType, 10, int, range(1,51)),
            })

    def _get_query(self):
        return self.get_param("query")
    def _set_query(self, value):
        self.set_param("query",value)
    query = property(_get_query, _set_query, None,
                     "The query to get related searches from")

    def _get_results(self):
        return self.get_param("results")
    def _set_results(self, value):
        self.set_param("results", value)
    results = property(_get_results, _set_results, None,
                       "The number of results to return")


class SpellingSuggestion(_Search):
    """SpellingSuggestion - perform a Yahoo Web Spelling Suggestion search

    This class implements the Web Search Spelling Suggestion web service
    APIs. The only allowed parameter is:

        query        - The query to get spelling suggestions for

    Full documentation for this service is available at:

        http://developer.yahoo.net/web/V1/spellingSuggestion.html
    """
    NAME = "spellingSuggestion"
    SERVICE = "WebSearchService"
    _RESULT_FACTORY = domparsers.SpellingSuggestion

    def _init_valid_params(self):
        self._valid_params.update({
            "query" : (types.StringTypes, None, None,None),
            })

    def _get_query(self):
        return self.get_param("query")
    def _set_query(self, value):
        self.set_param("query",value)
    query = property(_get_query, _set_query, None,
                     "The query to get spelling suggestions for")


class NewsSearch(_BasicParams, _Search):
    """NewsSearch - perform a Yahoo News Search

    This class implements the News Search web service APIs. Allowed
    parameters are:
    
        query        - The query to search for
        type         - The kind of search to submit (all, any or phrase)
        results      - The number of results to return
        start        - The starting result position. This number should
                       increment by the value of the results argument
                       each time the user chooses the next page of results
        sort         - Sort articles by relevance ('rank') or most-recent
                       ('date'). Default is by relevance
        language     - The language the results are written in

    Full documentation for this service is available at:

        http://developer.yahoo.net/news/V1/newsSearch.html
    """
    NAME = "newsSearch"
    SERVICE = "NewsSearchService"
    _RESULT_FACTORY = domparsers.NewsSearch

    def _init_valid_params(self):
        super(NewsSearch, self)._init_valid_params()
        self._valid_params.update({
            "type" : (types.StringTypes, "all", string.lower,
                      ("all", "any", "phrase")),
            "sort" : (types.StringTypes, "rank", string.lower, ("date", "rank")),
            "language" : (types.StringTypes, "en", string.lower, LANGUAGES),
            })

    def _get_type(self):
        return self.get_param("type")
    def _set_type(self, value):
        self.set_param("type", value)
    type = property(_get_type, _set_type, None,
"""The kind of search to submit:

    * all returns results with all query terms
    * any returns results with one or more of the query terms
    * phrase returns results containing the query terms as a phrase""")

    def _get_sort(self):
        return self.get_param("sort")
    def _set_sort(self, value):
        self.set_param("sort", value)
    sort = property(_get_sort, _set_sort, None,
"""Sort articles by relevance ('rank') or most-recent ('date').
Default is by relevance""")

    def _get_language(self):
        return self.get_param("language")
    def _set_language(self, value):
        self.set_param("language", value)
    language = property(_get_language, _set_language, None,
                        "The language the results are written in")


class LocalSearch(_BasicParams, _Search):
    """LocalSearch - perform a Yahoo Local Search

    This class implements the Local Search web service APIs. Allowed
    parameters are:
    
        query        - The query to search for
        results      - The number of results to return (1-20)
        start        - The starting result position. This number should
                       increment by the value of the results argument
                       each time the user chooses the next page of results
        radius       - How far from the specified location to search for
                       the query terms
        street       - Street name. The number is optional
        city         - City name
        state        - The United States state. You can spell out the
                       full state name or you can use the two-letter
                       abbreviation
        zip          - The five-digit zip code, or the five-digit code
                       plus four-digit extension
        location     - Free form field for location (see below)
                        
    Full documentation for this service is available at:

        http://developer.yahoo.net/local/V1/localSearch.html
    """
    NAME = "localSearch"
    SERVICE = "LocalSearchService"
    SERVER = "api.local.yahoo.com"
    _RESULT_FACTORY = domparsers.LocalSearch

    def _init_valid_params(self):
        super(LocalSearch, self)._init_valid_params()
        self._valid_params.update({
            "results" : (types.IntType, 10, int, range(1,21)),
            "radius" : (types.FloatType, None, float, None),
            "street" : (types.StringTypes, None, None, None),
            "city" : (types.StringTypes, None, None, None),
            "state" : (types.StringTypes, None, None, None),
            "zip" : (types.StringTypes, None, None, None),
            "location" : (types.StringTypes, None, None, None),
            })


    def _get_radius(self):
        return self.get_param("radius")
    def _set_radius(self, value):
        self.set_param("radius", value)
    radius = property(_get_radius, _set_radius, None,
                    "How far from the specified location to search for the query")

    def _get_street(self):
        return self.get_param("street")
    def _set_street(self, value):
        self.set_param("street", value)
    street = property(_get_street, _set_street, None,
                      " Street name. The number is optional")

    def _get_city(self):
        return self.get_param("city")
    def _set_city(self, value):
        self.set_param("city", value)
    city = property(_get_city, _set_city, None,
                    "City name")

    def _get_state(self):
        return self.get_param("state")
    def _set_state(self, value):
        self.set_param("state", value)
    state = property(_get_state, _set_state, None,
"""The United States state. You can spell out the full state name or
you can use the two-letter abbreviation""")

    def _get_zip(self):
        return self.get_param("zip")
    def _set_zip(self, value):
        self.set_param("zip", value)
    zip = property(_get_zip, _set_zip, None,
"""The five-digit zip code, or the five-digit code plus four-digit
extension. If this location contradicts the city and state,
specified the zip code takes priority""")

    def _get_location(self):
        return self.get_param("location")
    def _set_location(self, value):
        self.set_param("location", value)
    location = property(_get_location, _set_location, None,
"""This free field lets users enter any of the following:

    * city, state
    * city, state, zip
    * zip
    * street, city, state
    * street, city, state, zip
    * street, zip""")


#
# This is a "convenience" dictionary, providing a (very) short
# description of all available Search classes. The factory function
# uses this dictionary to make a "simple" interface to instantiate
# and configure a search object in one call.
#
SERVICES = {"Video" : (VideoSearch, "Video Search"),
            "Image" : (ImageSearch, "Image Search"),
            "Web" : (WebSearch, "Web Search"),
            "News" : (NewsSearch, "News Search"),
            "Local" : (LocalSearch, "Local Search"),
            "Related" : (RelatedSuggestion, "Web Search Related Suggestion"),
            "Spelling" : (SpellingSuggestion, "Web Search Spelling Suggestion"),
            }

def create_search(name, app_id, xml_parser=None, result_factory=None, debug=0, **args):
    """Create a Yahoo Web Services object, and configure it

    This is a simple "factory" function to instantiate and configure
    a Yahoo Web Services object. For example:

        app_id = "something"
        srch = create_search("Web", app_id, query="Leif Hedstrom", results=4)
        dom = srch.get_results()

    The first argument is one of the following "classes" of searches:

        Web	 - Web search
        Video	 - Video search
        Image	 - Image search
        News	 - News search
        Local	 - Local search
        Related	 - Web search Related Suggestions
        Spelling - Web search Spelling Suggestions

    The second argument, app_id, is an application specific identifier,
    provided by Yahoo. The web services will not accept requests without
    a proper AppID.

    All other arguments must be valid named arguments, and the allowed
    set of parameters depends on the specific class of search being
    instantiated. See <XXX TODO: URL here?> for a comprehensive list and
    documentation of allowed parameters for all types of searches.
    """

    if not name.istitle():
        name = name.title()
    if not SERVICES.has_key(name):
        return None

    obj = SERVICES[name][0](app_id, xml_parser=xml_parser,
                            result_factory=result_factory, debug=debug)
    if obj and args:
        obj.set_params(args)
    return obj



#
# local variables:
# mode: python
# indent-tabs-mode: nil
# py-indent-offset: 4
# end:
